#include "FsFilter.h"

PFLT_FILTER gFilterHandle;

CONST FLT_OPERATION_REGISTRATION Callbacks[] = {
    { 
		IRP_MJ_CREATE,
		0,
		FsFilterPreOperation,
		NULL
	},

    { IRP_MJ_OPERATION_END }
};


CONST FLT_REGISTRATION FilterRegistration = {
    sizeof(FLT_REGISTRATION), //  Size
    FLT_REGISTRATION_VERSION, //  Version
    0,                        //  Flags

    NULL,                     //  Context
    Callbacks,                //  Operation callbacks

    FsFilterUnload,           //  MiniFilterUnload

    FsFilterLoad,	//  InstanceSetup
    NULL,	//  InstanceQueryTeardown
    NULL,	//  InstanceTeardownStart
    NULL,	//  InstanceTeardownComplete

    NULL,                               //  GenerateFileName
    NULL,                               //  GenerateDestinationFileName
    NULL                                //  NormalizeNameComponent
};


/*************************************************************************
    MiniFilter initialization and unload routines.
*************************************************************************/

NTSTATUS DriverEntry(IN PDRIVER_OBJECT DriverObject, IN PUNICODE_STRING RegistryPath) {
    NTSTATUS status;

    UNREFERENCED_PARAMETER(RegistryPath);

    DbgPrint("FsFilter!DriverEntry\r\n");

    status = FltRegisterFilter(DriverObject, &FilterRegistration, &gFilterHandle);

    if (NT_SUCCESS(status)) {
        status = FltStartFiltering(gFilterHandle);
		DbgPrint("FsFilter!FltStartFiltering: 0x%X\r\n", status);

        if (!NT_SUCCESS(status)) {
			DbgPrint("FsFilter!FltUnregisterFilter\r\n");
			FltUnregisterFilter(gFilterHandle);
        }
    }

	return status;
}


NTSTATUS FsFilterLoad(
	IN PCFLT_RELATED_OBJECTS  FltObjects,
	IN FLT_INSTANCE_SETUP_FLAGS  Flags,
	IN DEVICE_TYPE  VolumeDeviceType,
	IN FLT_FILESYSTEM_TYPE  VolumeFilesystemType
) {
	UNREFERENCED_PARAMETER(FltObjects);
	UNREFERENCED_PARAMETER(Flags);
	UNREFERENCED_PARAMETER(VolumeFilesystemType);

	PAGED_CODE();

	DbgPrint("FsFilter!FsFilterLoad\r\n");

	return VolumeDeviceType == FILE_DEVICE_NETWORK_FILE_SYSTEM ? 
		STATUS_FLT_DO_NOT_ATTACH : 
		STATUS_SUCCESS;
}


NTSTATUS FsFilterUnload(IN FLT_FILTER_UNLOAD_FLAGS Flags) {
    UNREFERENCED_PARAMETER(Flags);
    PAGED_CODE();
	FltUnregisterFilter(gFilterHandle);
    DbgPrint("FsFilter!FsFilterUnload\r\n");
    return STATUS_SUCCESS;
}


/*************************************************************************
    MiniFilter callback routines.
*************************************************************************/

FLT_PREOP_CALLBACK_STATUS FsFilterPreOperation(
	IN OUT PFLT_CALLBACK_DATA Data,
	IN PCFLT_RELATED_OBJECTS FltObjects,
	_Flt_CompletionContext_Outptr_ PVOID *CompletionContext
) {
	UNREFERENCED_PARAMETER(FltObjects);
	UNREFERENCED_PARAMETER(CompletionContext);

	PAGED_CODE();

	LPWSTR IopbFileName = Data->Iopb->TargetFileObject->FileName.Buffer;
	LPWSTR FltObjFileName = FltObjects->FileObject->FileName.Buffer;

	if (KeGetCurrentIrql() == PASSIVE_LEVEL) {
		DbgPrint("DataFileName = %ws\r\nFltObj = %ws\r\n\r\n", IopbFileName, FltObjFileName);
	}

    DbgPrint("FsFilter!FsFilterPreOperation\r\n");

    return FLT_PREOP_SUCCESS_NO_CALLBACK;
}